/*
*  Rubrica
*  file: file_utils.h
*  
*  Copyright (C) 2000-2003 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>
#include <string.h>
#include <time.h>
#include <libxml/tree.h>
#include <libxml/parser.h>

#include "rubrica_type.h"
#include "file_utils.h"
#include "dialog.h"
#include "prefer.h"
#include "utils.h"
#include "log.h"
#include "export.h"


/*  open the file "filename"
    return ptr to xml doc or NULL on error
 */
xmlDocPtr 
rubrica_file_utils_open_doc(gchar *filename)
{
  xmlDocPtr doc;

  g_return_val_if_fail(filename != NULL, FALSE);
  
  /* read and parse file 
   */
  doc = xmlParseFile(filename);

  if (!doc)
    {
      rubrica_dialog_error_loading_file(filename, RE_PARSE_XML_DOC);
      
      return NULL;
    }  

  return doc;
}


/*  get the file format of the rubrica's doc    
*/
gint 
rubrica_file_utils_get_fileformat(xmlDocPtr doc)
{
  if(!xmlHasProp(doc->children, "fileformat"))
    return rubrica_utils_str2int(NULL);

  return rubrica_utils_str2int(xmlGetProp(doc->children, "fileformat"));
}

/*  get version.
    the version of rubrica that make the file
*/
gchar*
rubrica_file_utils_get_version(xmlDocPtr doc)
{
  if(!xmlHasProp(doc->children, "version"))
    return NULL;

  return xmlGetProp(doc->children, "version");
}


/*  get the document type
    values: ADDRESSBOOK_TYPE_DOC
            ACCOUNT_TYE_DOC
*/
gchar*
rubrica_file_utils_get_document_tpye(xmlDocPtr doc)
{
  if(!xmlHasProp(doc->children, "doctype"))
    return NULL;

  return xmlGetProp(doc->children, "doctype");
}


/*  check if file "filename" is a rubrica doc
    return TRUE if xml doc is a rubrica doc FALSE otherwise
*/
gboolean 
rubrica_file_utils_is_rubrica_doc(xmlDocPtr doc, gchar *filename)
{
  g_return_val_if_fail(doc != NULL, FALSE);

  if (!doc->children || 
      !doc->children->name || 
      g_ascii_strcasecmp(doc->children->name, "Rubrica") != 0)
    {      
      rubrica_dialog_error_loading_file(filename, 
					RE_XML_DOC_NO_RUBRICA_HEADER);   
      xmlFreeDoc(doc);

      return FALSE;
    }  

  return TRUE;
}


gchar* 
rubrica_file_utils_get_node_content(xmlNodePtr node)
{
  xmlChar* tmp = NULL;

  if (!node)
    return NULL;
  
  tmp = xmlNodeGetContent(node);

  if (tmp)    
    {
#ifdef DEBUG
      g_print("\nnome letto %s", tmp);
#endif
      
      return (gchar *) tmp;
    }
  
  return NULL;
}


gchar* 
rubrica_file_utils_get_node_property(xmlNodePtr node, const gchar* key)
{
  xmlChar* prop;
  gchar* ret;

  if (!xmlHasProp(node, (xmlChar*) key))
    return NULL;

  prop = xmlGetProp(node, (xmlChar*) key);
  ret  = g_strdup(prop);
  xmlFree(prop);

#ifdef DEBUG
  g_print("\n*** DEBUG *** il nodo per la chiave %s ha valore %s", key, ret);
#endif 

  return ret;
}


gchar* 
rubrica_file_utils_get_card_name(xmlNodePtr node)
{
  return rubrica_file_utils_get_node_property(node, "name");
}


gboolean
rubrica_file_utils_get_boolean_property(xmlNodePtr node, gchar* key)
{
  gchar* tmp = NULL;
  gboolean ret = FALSE;

  tmp = rubrica_file_utils_get_node_property(node, key);

  if (tmp && xmlStrcmp(tmp, (xmlChar*) "true") == 0)
    ret = TRUE;
  
  g_free(tmp);
  return ret;
}


glong 
rubrica_file_utils_get_date_property (xmlNodePtr node, gchar* key)
{
  gchar* tmp = NULL;
  glong ret;

  tmp = rubrica_file_utils_get_node_property(node, key);
  ret = tmp ? atol(tmp) : time(NULL);
  g_free(tmp);

  return ret;
}


time_t
rubrica_file_utils_get_calendar_property(xmlNodePtr node)
{
  GDate *gdate;
  struct tm *tm;
  gchar *day, *month, *year;
  
  day   = rubrica_file_utils_get_node_property(node, "day");
  month = rubrica_file_utils_get_node_property(node, "month");
  year  = rubrica_file_utils_get_node_property(node, "year");
  
  if (day && month && year)
    {
      if (g_ascii_strcasecmp(day, "BadDay") == 0)
	goto exit;
      if (g_ascii_strcasecmp(month, "BadMonth") == 0)
	goto exit;	
      if (g_ascii_strcasecmp(year, "BadYear") == 0)
	goto exit;
      
      gdate = g_date_new_dmy(atoi(day), atoi(month), atoi(year));
      g_date_to_struct_tm(gdate, tm);
      g_date_free(gdate);

      return mktime(tm);
    }

 exit:
  if (day)   
    g_free(day);
  if (month) 
    g_free(month);
  if (year)  
    g_free(year);
  return time(NULL);
}


/*  check the "filename" extension
    return TRUE  if filename has extension .rub FALSE otherwise
*/
gboolean 
rubrica_file_utils_has_rub_extension(gchar *filename)
{
  gchar *tmp=NULL, *buf;
  gint i, len;

  i = 0;
  buf = g_path_get_basename(filename);
  len = strlen(buf);

  while(*buf != '.')
    {
      buf++;
      i++;
            
      if (i == len)     /* end buffer was reached and dot wasn't found */
	return FALSE;	
    }

  tmp = (gchar*) g_malloc(((len - i)*sizeof(gchar))+1);
  strcpy(tmp, buf);
  
  if ((strcmp(tmp,".rub")) == 0)
    {
      g_free(tmp);
      return TRUE;
    }
  else 
    g_free(tmp);

  g_free(buf);
  return FALSE;  
}



/*
 *
 *  Backup routines
 *
 */ 
void 
rubrica_file_utils_do_backup(RubricaDoc *doc, gchar *oldname)
{
  gint err;
  gchar *newname;
	
  if (backup_is_increasing())
    {
      newname = g_strdup_printf("%s1", oldname);
      
      /*  if exist a file with same name, then do a backup of old file 
       */
      if (g_file_test(newname, G_FILE_TEST_EXISTS))
	{
	  gchar *path;
	  gchar *ext=NULL, *f=NULL;
	  gchar *bfile;
	  gint num;
	  
	  path = g_path_get_dirname(newname);
	  rubrica_utils_file_split(newname, &f, &ext, &num);
	  
	  bfile = g_strdup_printf("%s/%s%d.%s", path, f, num, ext);
	  rubrica_file_utils_do_backup(doc, bfile);
	  
	  g_free(bfile);
	}
      else
	{
	  err = rename(oldname, newname);  
	  
	  if (!err)
	    {
	      gchar *str;
	      
	      str = g_strdup_printf("\nBackup -- old file: %s has "
				    "been saved in: %s", oldname, newname);
	      
	      rubrica_log_write_log(str);
	      g_free(str);
	    }
	  
	  else
	    {
	      gchar *str;
	      
	      str = g_strdup_printf("\nBackup -- This error occurred "
				    "during backup\nold file name: %s\n"
				    "new file name: %s\nerror: %s", 
				    oldname, newname, strerror(errno)); 
	      
	      rubrica_log_write_log(str);
	      g_free(str);
	    }	      
	}
      
      g_free(newname);
    }
  else
    rubrica_file_utils_do_bak_backup(doc, oldname);
}


void 
rubrica_file_utils_do_bak_backup(RubricaDoc *doc, gchar *fname)
{
  gchar *ext=NULL, *file=NULL, *new;
  gchar *path;
  gint num, err;
  
  path = g_path_get_dirname(fname);
  rubrica_utils_file_split(fname, &file, &ext, &num);	  
  
  new = g_strdup_printf("%s/%s.bak", path, file);
  err = rename(fname, new);  
  
  if (!err)
    {
      gchar *str;
      
      str = g_strdup_printf("\nBackup -- old file: %s has been saved in: %s", 
			    fname, new);
      
      rubrica_log_write_log(str);
      g_free(str);
    }
  
  g_free(new);  
}


/*  import an old rubrica doc
*/
gboolean 
rubrica_file_utils_import_old_file(gchar* fname)
{
  gchar* msg;
  gchar* stylesheet;
  gchar* filename;
  
  rubrica_log_write_log(_("\nImporting old rubrica document. Please wait..."));
  msg = g_strdup_printf(_("\nThe document %s will be moved to "
			  "file format #%d"), fname, RUBRICA_FILE_FORMAT); 
  rubrica_log_write_log(msg);
  g_free(msg);
  
  filename = g_strdup_printf("%s%s.tmpfile",
			     rubrica_preferences_get_rubrica_home(),
			     G_DIR_SEPARATOR_S);
  
  stylesheet = g_strdup_printf("%s%sstylesheet%srub2rub.xsl", 
			       RUBRICA_SCRIPT_DIR, 
			       G_DIR_SEPARATOR_S,
			       G_DIR_SEPARATOR_S); 
  
  if (!g_file_test(stylesheet, G_FILE_TEST_EXISTS))
    {
      rubrica_dialog_error(_("Can't find the stylesheet."
			     "\nI can't read the old file."));
      
      g_free(stylesheet);
      g_free(filename);
      
      return FALSE;
    }        
  
  if(!rubrica_export_run_engine(fname, filename, stylesheet))
    {
      /* error dialog shown in rubrica_export_engine */
      g_free(stylesheet);
      g_free(filename);
      
      return FALSE;
    }
  else
    {
      gchar *newname;

      newname = g_strdup_printf("%s.SAVE", fname);
      if (rename(fname, newname) == -1)
	{
	  msg = g_strdup_printf("\nRubrica ** exporting %s to %s failed", 
				fname, newname);
	  
	  rubrica_log_write_log(msg);
	  g_free(msg);
	  
	  return FALSE;
	}
      g_free(newname);
      
      remove(fname);
      if (-1 == rename(filename, fname))
	{
	  gchar *msg;
	  
	  msg = g_strdup_printf("\nRubrica ** renaming %s to %s failed", 
				filename, fname);
	  
	  rubrica_log_write_log(msg);
	  g_free(msg);
	  
	  return FALSE;
	}
      g_free(filename);
    }
  
  return TRUE;
}

gboolean 
rubrica_file_utils_load_ff3_doc (RubricaDoc *doc, RubricaErrorType *err)
{
  
  return FALSE;
}





