/*
*  Rubrica
*  file: card_item.c
*  
*  Copyright (C) Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <time.h>
#include <gnome.h>

#include "item.h"
#include "card.h"
#include "form_card.h"
#include "rubrica.h"
#include "dialog.h"
#include "utils.h"


static void rubrica_item_class_init (RubricaItemClass* klass);
static void rubrica_item_init       (RubricaItem* obj);

static void rubrica_item_dispose  (RubricaItem* obj);
static void rubrica_item_finalize (RubricaItem* obj);



void rubrica_item_build_node      (xmlNodePtr xmlnode,
					RubricaItem* item);
void rubrica_item_free_item       ();


struct _RubricaItemPrivate {
  gboolean dispose_has_run;
};




GType
rubrica_item_get_type()
{
  static GType rubrica_item_type = 0;
  
  if (!rubrica_item_type)
    {
      static const GTypeInfo rubrica_item_info =
	{
	  sizeof(RubricaItemClass),
	  NULL,
	  NULL,
	  (GClassInitFunc) rubrica_item_class_init,
	  NULL,
	  NULL,
	  sizeof(RubricaItem),
	  0,
	  (GInstanceInitFunc) rubrica_item_init
	};

      rubrica_item_type = g_type_register_static (G_TYPE_OBJECT,
						       "RubricaItem",
						       &rubrica_item_info,
						       0);
    }

  return rubrica_item_type;
}


static void
rubrica_item_class_init(RubricaItemClass* klass)
{
  GObjectClass *class;
  RubricaDocClass* doc_class;
  
  class = G_OBJECT_CLASS (klass);
  doc_class = RUBRICA_DOC_GET_CLASS(klass);

  class->dispose  = rubrica_item_dispose;
  class->finalize = rubrica_item_finalize;
}


static void 
rubrica_item_dispose  (RubricaItem* card)
{
  if (card->private->dispose_has_run)
    return;

  card->private->dispose_has_run = TRUE;
}


static void 
rubrica_item_finalize (RubricaItem* item)
{
  GList* lst;
  GList* link;

  rubrica_free_string(item->card);
  rubrica_free_string(item->personal.first);
  rubrica_free_string(item->personal.middle);
  rubrica_free_string(item->personal.last);
  rubrica_free_string(item->personal.prof);
  rubrica_free_string(item->personal.prefix);
  rubrica_free_string(item->personal.title);

  rubrica_item_free_addresses(item->addresses);

  for (lst = item->web; lst; lst = g_list_next(lst))
    {    
      link = lst;
      lst  = g_list_remove_link(lst, link);
      rubrica_free_string(link->data);
      g_list_free_1(link);
    }
    
  for (lst = item->email; lst; lst = g_list_next(lst))
    {
      link = lst;
      lst  = g_list_remove_link(lst, link);
      rubrica_free_string(lst->data);  
      g_list_free_1(link);
    }
  
  for (lst = item->phone; lst; lst = g_list_next(lst))
    {
      RubricaCardTelNum *tn;
      
      link = lst;
      lst  = g_list_remove_link(lst, link);
      if ((tn = lst->data))
	{
	  rubrica_free_string(tn->type);
	  rubrica_free_string(tn->number);      
	}
      g_list_free_1(link);
    }

  rubrica_free_string(item->company.name);
  rubrica_free_string(item->company.street);
  rubrica_free_string(item->company.number);
  rubrica_free_string(item->company.zip);
  rubrica_free_string(item->company.city);
  rubrica_free_string(item->company.province);
  rubrica_free_string(item->company.country);
  rubrica_free_string(item->company.web);
  rubrica_free_string(item->company.email);
  rubrica_free_string(item->company.operator);
  rubrica_free_string(item->company.fax);
  rubrica_free_string(item->company.green);
  rubrica_free_string(item->company.ccare);
  rubrica_free_string(item->company.notes);
  
  rubrica_free_string(item->work.assigment);
  rubrica_free_string(item->work.org);
  rubrica_free_string(item->work.dep);
  rubrica_free_string(item->work.subdep);
  rubrica_free_string(item->work.secretary);
  rubrica_free_string(item->work.telephone);

  rubrica_free_string(item->notes.spouse);
  rubrica_free_string(item->notes.children);
  rubrica_free_string(item->notes.notes);
  rubrica_free_string(item->notes.pubkey);

  g_free(item);  
}


static void
rubrica_item_init(RubricaItem* item)
{
  item->id        = 0L;

  item->deletable = TRUE;
  item->deleted   = FALSE;
  item->marked    = FALSE;
  item->created   = item->changed = time(NULL);

  item->rank      = RUBRICA_RANK_NORMAL;
  item->groups    = NULL;
  item->card      = NULL;

  item->personal.first    = item->personal.middle = 
    item->personal.last   = item->personal.nick   = 
    item->personal.prof   = item->personal.prefix = 
    item->personal.title  = item->personal.photo  = NULL;

  item->personal.birth    = FALSE;
  item->personal.bdate    = time(NULL);
      
  item->addresses         = NULL;
  item->web               = NULL;
  item->email             = NULL;
  item->phone             = NULL;

  item->company.name      = item->company.street   =
    item->company.number  = item->company.zip      =
    item->company.city    = item->company.province = 
    item->company.country = item->company.web      = 
    item->company.email   = item->company.operator =
    item->company.fax     = item->company.green    = 
    item->company.ccare   = item->company.notes    = NULL;
  
  item->work.assigment    = item->work.org       =
    item->work.dep        = item->work.subdep    = 
    item->work.secretary  = item->work.telephone = NULL;
  
  item->notes.married     = item->notes.birth = item->notes.ann  = FALSE;
  item->notes.bdate       = item->notes.adate = time(NULL);
  item->notes.spouse      = item->notes.children = 
    item->notes.notes     = item->notes.pubkey   = NULL;

  item->private = g_new(RubricaItemPrivate, 1);
  item->private->dispose_has_run = FALSE;
}



RubricaItem* 
rubrica_item_new(RubricaDoc* doc)
{
  RubricaItem *item;

  item = g_object_new(rubrica_item_get_type(), NULL);

  RUBRICA_ITEM(item)->id = rubrica_doc_get_id(doc);
  item->father = doc;
  
  return (RubricaItem*) item;
}

gulong 
rubrica_item_get_id (RubricaItem* item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), 0L);

  return item->id;
}

void   
rubrica_item_set_id (RubricaItem* item, gulong id)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));
  
  item->id = (gulong) id; 
}

gboolean 
rubrica_item_is_deletable (RubricaItem* item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), FALSE);
  
  return (gboolean) item->deletable;
}

gboolean 
rubrica_item_is_deleted (RubricaItem* item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), FALSE);

  return (gboolean) item->deleted;
}

gboolean 
rubrica_item_is_marked (RubricaItem* item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), FALSE);

  return (gboolean) item->marked;
}


time_t   
rubrica_item_get_creation_time (RubricaItem* item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), 0);

  return item->created;
}

time_t   
rubrica_item_get_modification_time (RubricaItem* item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), 0);

  return item->changed;
}

gint 
rubrica_item_get_position(RubricaItem *item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), -1);

  return item->position;
}



RubricaItem*
rubrica_item_modify_item (RubricaDoc* doc, RubricaItem* item)
{
  gint index; 
  RubricaItem* new;

  g_return_val_if_fail(IS_RUBRICA_DOC(doc), NULL);
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), NULL);

  index = rubrica_doc_get_index(doc);
  new   = rubrica_form_card_modify_item(doc, item);

  if (new)
    {
      rubrica_card_modify_item(doc, new, index);
      
      return new;
    }

  /* else */
  rubrica_dialog_message(_("I can't get your changes to card"), 
			 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK); 	
  return NULL; 
}


RubricaItem*          
rubrica_item_retrieve_item (RubricaDoc* doc)
{
  return rubrica_form_card_get_item(doc);
}

void
rubrica_item_cut_item (RubricaDoc* doc)
{
  rubrica_card_cut_item(doc);
}


RubricaItem *
rubrica_item_copy_item(RubricaItem *item)
{
  RubricaItem *retval;
  GList* alias;

  g_return_val_if_fail(item != NULL, NULL);

  retval = rubrica_item_new(item->father); 
  if (!retval)
    {
      rubrica_dialog_error(_("I have not enough memory to perform this task.\n"
			     "I cant copy the selected item"));     
      
      return NULL;
    }
  
  retval->groups    = retval->addresses = NULL;
  retval->web       = retval->email     = retval->phone = NULL;

  retval->father    = NULL;
  RUBRICA_ITEM(retval)->deletable = RUBRICA_ITEM(item)->deletable;
  RUBRICA_ITEM(retval)->deleted   = RUBRICA_ITEM(item)->deleted;
  RUBRICA_ITEM(retval)->marked    = RUBRICA_ITEM(item)->marked;
  retval->rank      = item->rank;

  retval->card = (item->card ? g_strdup(item->card) : "");
  
  retval->personal.first  = (item->personal.first  ? 
			     g_strdup(item->personal.first) : "");
  retval->personal.middle = (item->personal.middle ? 
			     g_strdup(item->personal.middle): "");
  retval->personal.last   = (item->personal.last   ? 
			     g_strdup(item->personal.last): "");
  retval->personal.nick   = (item->personal.nick   ? 
			     g_strdup(item->personal.nick): "");
  retval->personal.prof   = (item->personal.prof ? 
			     g_strdup(item->personal.prof): "");
  retval->personal.prefix = (item->personal.prefix ? 
			     g_strdup(item->personal.prefix): "");
  retval->personal.title  = (item->personal.title ? 
			     g_strdup(item->personal.title): "");
  retval->personal.birth  = item->personal.birth;
  retval->personal.bdate  = item->personal.bdate;
  
  alias = item->groups;
  for (; alias; alias = g_list_next(alias))
    {
      RubricaGroup* new;  
      RubricaGroup* group;

      new   = g_malloc(sizeof(RubricaGroup));
      group = alias->data;

      if (group && new)
	{
	  new->label  = g_strdup(group->label);
	  new->type   = group->type;
	  
	  retval->groups = g_list_append(retval->groups, new);
	}
    }

  alias = item->addresses;
  for (; alias; alias = g_list_next(alias))
    {
      RubricaCardAddress* new;
      RubricaCardAddress* address;

      new = (RubricaCardAddress *) g_malloc(sizeof(RubricaCardAddress));
      address = alias->data;

      if (address && new)
	{
	  new->type     = address->type;
	  new->street   = (address->street   ? g_strdup(address->street): "");
	  new->number   = (address->number   ? g_strdup(address->number): "");
	  new->zip      = (address->zip      ? g_strdup(address->zip):    "");
	  new->city     = (address->city     ? g_strdup(address->city):   "");
	  new->province = (address->province ? g_strdup(address->province):"");
	  new->state    = (address->state    ? g_strdup(address->state):   "");
	  new->country  = (address->country  ? g_strdup(address->country): "");
	  
	  retval->addresses = g_list_append(retval->addresses, new);  
	}
    }  

  alias = item->web;
  for (; alias; alias = g_list_next(alias))
    {
      gchar *web;

      if (alias->data)
	{
	  web = g_strdup(alias->data);
	  retval->web = g_list_append(retval->web, web);
	}
    }

  alias = item->email;
  for (; alias; alias = g_list_next(alias))    
    {
      gchar *email;

      if (alias->data)
	{
	  email = g_strdup(alias->data);
	  retval->email = g_list_append(retval->email, email);
	}
    }  

  alias = item->phone;
  for (; alias; alias = g_list_next(alias))
    {
      RubricaCardTelNum *tn;
      RubricaCardTelNum *tel;

      tn  = (RubricaCardTelNum *) g_malloc(sizeof(RubricaCardTelNum));
      tel = alias->data;
      
      if (tel && tn)
	{
	  tn->number = g_strdup(tel->number);
	  tn->type   = g_strdup(tel->type);
	  retval->phone = g_list_append(retval->phone, tn);
	}
    }
  
  retval->work.assigment   = (item->work.assigment  ?
			      g_strdup(item->work.assigment): "");
  retval->work.org         = (item->work.org ? g_strdup(item->work.org): "");
  retval->work.dep         = (item->work.dep ? g_strdup(item->work.dep): "");
  retval->work.subdep      = (item->work.subdep ? 
			      g_strdup(item->work.subdep): "");
  retval->work.secretary   = (item->work.secretary ? 
			       g_strdup(item->work.secretary): "");
  retval->work.telephone   = (item->work.telephone ? 
			       g_strdup(item->work.telephone): "");
  
  retval->company.name     = (item->company.name     ? 
			      g_strdup(item->company.name): "");
  retval->company.street   = (item->company.street   ? 
			      g_strdup(item->company.street): "");
  retval->company.number   = (item->company.number   ? 
			      g_strdup(item->company.number): "");
  retval->company.zip      = (item->company.zip      ? 
			      g_strdup(item->company.zip): "");
  retval->company.city     = (item->company.city     ? 
			      g_strdup(item->company.city): "");
  retval->company.province = (item->company.province ? 
			      g_strdup(item->company.province): "");
  retval->company.country  = (item->company.country  ? 
			      g_strdup(item->company.country): "");
  retval->company.web      = (item->company.web      ? 
			      g_strdup(item->company.web): "");
  retval->company.email    = (item->company.email    ? 
			      g_strdup(item->company.email): "");
  retval->company.operator = (item->company.operator ? 
			      g_strdup(item->company.operator): "");
  retval->company.fax      = (item->company.fax      ? 
			      g_strdup(item->company.fax): "");
  retval->company.green    = (item->company.green    ? 
			      g_strdup(item->company.green): "");
  retval->company.ccare    = (item->company.ccare ? 
			      g_strdup(item->company.ccare): "");
  retval-> company.notes   = (item->company.notes ? 
			      g_strdup(item->company.notes): "");
  
  retval->notes.married = item->notes.married;
  retval->notes.birth   = item->notes.birth;  
  retval->notes.ann     = item->notes.ann;  

  retval->notes.bdate   = item->notes.bdate;  
  retval->notes.adate   = item->notes.adate;  

  retval->notes.spouse   = (item->notes.spouse ? 
			    g_strdup(item->notes.spouse): "");
  retval->notes.children = (item->notes.children ? 
			    g_strdup(item->notes.children): "");
  retval->notes.notes    = (item->notes.notes ? 
			    g_strdup(item->notes.notes): "");
  retval->notes.pubkey   = (item->notes.pubkey ? 
			    g_strdup(item->notes.pubkey): "");
  
  RUBRICA_ITEM(retval)->created = RUBRICA_ITEM(item)->created;
  RUBRICA_ITEM(retval)->changed = RUBRICA_ITEM(item)->changed;

  return retval;
}



RubricaItem*
rubrica_item_paste_item()
{
  return NULL;
}


void           
rubrica_item_build_xmltree (RubricaDoc* doc)
{
  xmlDocPtr xmldoc;     /* pointer to xml doc */
  GList* itms;
  gchar *fileformat;

  xmldoc = xmlNewDoc("1.0");  
  fileformat = g_strdup_printf("%d", RUBRICA_FILE_FORMAT);

  xmldoc->children = xmlNewDocNode(xmldoc, NULL, "Rubrica", NULL);
  xmlSetProp(xmldoc->children, "version", RUBRICA_VERSION);
  xmlSetProp(xmldoc->children, "fileformat", fileformat); 
  xmlSetProp(xmldoc->children, "doctype", "AddressBook"); 
  g_free(fileformat);  
  
  /* browse addressbook and write data in xml tree 
   */
  for (itms = doc->docs; itms; itms = itms->next)
    {
      RubricaItem *item;
      
      item = (RubricaItem *) itms->data;
      
      if (item)
	rubrica_item_build_node(xmldoc->children, item);
    }

  doc->xmldoc = xmldoc;
}

void 
write_str_prop(xmlNodePtr node, gchar *label, gchar* string)
{
  xmlChar* xmlstr;
  
  g_return_if_fail(node != NULL);
  g_return_if_fail(label != NULL);
  
  xmlstr = xmlStrdup(string);
  
  xmlNewProp(node, label, xmlstr);
}


void 
write_bool_prop(xmlNodePtr node, gchar *label, gboolean bool)
{
  xmlChar* xmlstr;
  
  g_return_if_fail(node != NULL);
  g_return_if_fail(label != NULL);
  
  xmlstr = xmlStrdup((bool ? "true" : "false"));
  
  xmlNewProp(node, label, xmlstr);
}

void 
write_num_prop(xmlNodePtr node, gchar *label, gulong val)
{
  xmlChar* xmlstr;
  gchar* str;
  
  g_return_if_fail(node != NULL);
  g_return_if_fail(label != NULL);
  
  str  = g_strdup_printf("%ld", val);
  xmlstr = xmlStrdup(str);
  
  xmlNewProp(node, label, xmlstr);
  g_free(str);
}

void 
write_date_prop(xmlNodePtr node, gboolean bool, time_t date)
{
  GDate* gdate;
  GDateDay day;
  GDateMonth month;
  GDateYear year;
  
  g_return_if_fail(node != NULL);
  
  write_bool_prop(node, "known", bool);

  gdate = g_date_new();
  g_date_set_time(gdate, date);
  day   = g_date_get_day(gdate);
  month = g_date_get_month(gdate);
  year  = g_date_get_year(gdate);
  
  if (!bool || day == G_DATE_BAD_DAY)  
    write_str_prop(node, "day", "BadDay");
  else
    write_num_prop(node, "day", day);
  
  if (!bool || month == G_DATE_BAD_MONTH)
    write_str_prop(node, "month", "BadMonth");
  else
    write_num_prop(node, "month", month);
  
  if (!bool || year == G_DATE_BAD_YEAR)
    write_str_prop(node, "year", "BadYear");
  else
    write_num_prop(node, "year", year);
  
  g_date_free(gdate);
}


void 
rubrica_item_build_node(xmlNodePtr xmlnode, RubricaItem* item)
{
  xmlNodePtr cardxml;     /* card node */

  xmlNodePtr groupxml;    /* group node                     */
  xmlNodePtr datixml;     /* personal data node             */
  xmlNodePtr addressxml;  /* addresses node, address father */ 
  xmlNodePtr adsxml;      /* address node, addresses child  */
  xmlNodePtr webxml;      /* web node       */
  xmlNodePtr mailxml;     /* email node     */
  xmlNodePtr telxml;      /* telephone node */
  xmlNodePtr companyxml;  /* company node   */
  xmlNodePtr workxml;     /* work node      */
  xmlNodePtr notexml;     /* note node      */
  xmlNodePtr node;

  RubricaGroupList groups;
  GList* alias;

  g_return_if_fail(xmlnode != NULL);
  g_return_if_fail(item != NULL);

  groups = rubrica_item_get_groups(item);

  /* card xmlnode: 
     is a Rubrica's child 
     is the father of groups, address, webaddresses, etc
  */
  cardxml = xmlNewTextChild(xmlnode, NULL, "Card", NULL);

  /* set card property:
     name:        card name
     deletable:   is a deletable card
     deleted:     card was deleted (when deleted, card is only marked as 
                  deleted real deleting is done during addressbook 
                  optimization)
     rank:        card rank
     id:          card id
     created:     time when card was created
     last_change: last time card was changed
  */
  write_str_prop (cardxml, "name",        item->card);
  write_bool_prop(cardxml, "deletable",   item->deletable);
  write_bool_prop(cardxml, "deleted",     item->deleted);
  write_num_prop (cardxml, "rank",        item->rank);
  write_num_prop (cardxml, "id",          item->id);
  write_num_prop (cardxml, "created",     item->created);
  write_num_prop (cardxml, "last_change", item->changed);
  
  /*    Groups
  */
  groupxml = xmlNewTextChild(cardxml, NULL, "Groups", NULL);
  while(groups)
    {
      if (groups->data)
	xmlNewTextChild(groupxml, NULL, "Group", 
			rubrica_item_get_group_label(groups->data));  

      groups = g_list_next(groups);
    }
 
  /*    Personal 
   */
  datixml = xmlNewTextChild(cardxml, NULL, "Data", NULL);
  xmlNewTextChild(datixml, NULL, "FirstName",  item->personal.first);
  xmlNewTextChild(datixml, NULL, "MiddleName", item->personal.middle);
  xmlNewTextChild(datixml, NULL, "LastName",   item->personal.last);
  xmlNewTextChild(datixml, NULL, "Nickname",   item->personal.nick);
  xmlNewTextChild(datixml, NULL, "Profession", item->personal.prof);
  xmlNewTextChild(datixml, NULL, "NamePrefix", item->personal.prefix);
  xmlNewTextChild(datixml, NULL, "Title",      item->personal.title);
  node = xmlNewTextChild(datixml, NULL, "Birthday", NULL);
  write_date_prop(node, item->personal.birth, item->personal.bdate);

  /*    Addresses 
   */
  addressxml = xmlNewTextChild(cardxml, NULL, "Addresses", NULL);
  for (alias = item->addresses; alias; alias = g_list_next(alias))
    {
      RubricaCardAddress* address;
      xmlNodePtr node;  /* street node */
  
      if ((address = alias->data))
	{
	  adsxml = xmlNewTextChild(addressxml, NULL, "Address", NULL);
	  write_str_prop(adsxml, "type", RUBRICA_ADDRESS_TYPE(address->type));

	  node = xmlNewTextChild(adsxml, NULL, "Street", address->street);
	  write_str_prop(node, "number", address->number);

	  node = xmlNewTextChild(adsxml, NULL, "City",  address->city);
	  write_str_prop(node, "zip", address->zip);
	  
	  xmlNewTextChild(adsxml, NULL, "Province",     address->province);
	  xmlNewTextChild(adsxml, NULL, "State",        address->state);
	  xmlNewTextChild(adsxml, NULL, "Country",      address->country);
	}
    }

  /*    Net 
   */
  webxml = xmlNewTextChild(cardxml, NULL, "WebAddresses", NULL);
  for (alias = item->web; alias; alias = g_list_next(alias))
    if (alias && alias->data)
      xmlNewTextChild(webxml, NULL, "Web", alias->data);
  
  mailxml = xmlNewTextChild(cardxml, NULL, "EmailAddresses", NULL);  
  for (alias = item->email; alias; alias = g_list_next(alias))
    if (alias && alias->data)
      xmlNewTextChild(mailxml, NULL, "Email", alias->data);
  
  /*    Telephone 
   */
  telxml = xmlNewTextChild(cardxml, NULL, "TelephoneNumbers", NULL);  
  for (alias = item->phone; alias; alias = g_list_next(alias))
    if (alias && alias->data)
      {
	RubricaCardTelNum *tn = (RubricaCardTelNum*) alias->data;
	
	node = xmlNewTextChild(telxml, NULL, "Telephone", tn->number);
	write_str_prop(node, "type", tn->type);
      }

  /*    Company 
  */
  companyxml = xmlNewTextChild(cardxml, NULL, "Company", NULL);

  xmlNewTextChild(companyxml, NULL, "CompanyName",   item->company.name);
  node = xmlNewTextChild(companyxml, NULL, "Street", item->company.street);
  xmlNewProp(node, "number", item->company.number);
  node = xmlNewTextChild(companyxml, NULL, "City",   item->company.city);
  xmlNewProp(node, "zip", item->company.zip);
  xmlNewTextChild(companyxml, NULL, "Province",      item->company.province);
  xmlNewTextChild(companyxml, NULL, "Country",       item->company.country);
  xmlNewTextChild(companyxml, NULL, "Web",           item->company.web);
  xmlNewTextChild(companyxml, NULL, "Email",         item->company.email);
  xmlNewTextChild(companyxml, NULL, "Operator",      item->company.operator);
  xmlNewTextChild(companyxml, NULL, "Fax",           item->company.fax);
  xmlNewTextChild(companyxml, NULL, "Green",         item->company.green);
  xmlNewTextChild(companyxml, NULL, "CustomerCare",  item->company.ccare);
  xmlNewTextChild(companyxml, NULL, "Notes",         item->company.notes);
  
  /*    Work 
   */
  workxml = xmlNewTextChild(cardxml, NULL, "Work", NULL);

  xmlNewTextChild(workxml, NULL, "Assigment",      item->work.assigment);
  xmlNewTextChild(workxml, NULL, "Organization",   item->work.org);
  xmlNewTextChild(workxml, NULL, "Department",     item->work.dep);
  xmlNewTextChild(workxml, NULL, "SubDepartment",  item->work.subdep);
  xmlNewTextChild(workxml, NULL, "SecretaryName",  item->work.secretary);
  xmlNewTextChild(workxml, NULL, "SecretaryPhone", item->work.telephone);


  /*    Notes 
  */
  notexml = xmlNewTextChild(cardxml, NULL, "Notes", NULL);
  write_bool_prop(notexml, "married", item->notes.married);
  xmlNewTextChild(notexml, NULL, "SpouseName", item->notes.spouse);

  node = xmlNewTextChild(notexml, NULL, "SpouseBirthday", NULL);
  write_date_prop(node, item->notes.birth, item->notes.bdate);
  
  node = xmlNewTextChild(notexml, NULL, "Anniversary", NULL);
  write_date_prop(node, item->notes.ann, item->notes.adate);

  xmlNewTextChild(notexml, NULL, "Child",      item->notes.children);
  xmlNewTextChild(notexml, NULL, "OtherNotes", item->notes.notes);
  xmlNewTextChild(notexml, NULL, "PublicKey",  item->notes.pubkey);  
}



/*
void             
rubrica_item_delete (RubricaItem* item)
{
  g_return_if_fail(item!= NULL);

  if (rubrica_item_is_deletable(item))
    {
      rubrica_item_set_deleted(item, TRUE);
      rubrica_doc_delete_item(item->father);
    }
}

*/

/*
void 
rubrica_item_erase_item (RubricaItem* item)
{
  RubricaDoc* doc;
  GList* node;

  g_return_if_fail(item != NULL);

  doc = item->father;
  node = g_list_find(doc->docs, item);
*/
  /* TO DO: cancellare il nodo dalla lista 
            liberare la memoria occupata dal nodo   
  */
/*
  rubrica_doc_delete_item(doc);
}
*/


/* does item need id? */
gboolean         
rubrica_item_need_id (RubricaItem *item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), FALSE);

  return (gboolean) (RUBRICA_ITEM(item)->id == 0);
}


/* ask id to father */
gulong 
rubrica_item_ask_id (RubricaItem *item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), 0L);

  return rubrica_doc_get_id(item->father);
}

void 
rubrica_item_reject_id(RubricaItem *item)
{
  rubrica_doc_reject_id(item->father);
}


/* get item's father 
 */
RubricaDoc*      
rubrica_item_get_father (RubricaItem *item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), NULL);
  
  return (RubricaDoc*) item->father;
}


/* get item's rank 
 */
RubricaRankType 
rubrica_item_get_rank(RubricaItem *item)     
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), RUBRICA_RANK_NORMAL);

  return (gboolean) item->rank;
}


/* get group list whitch item belong 
 */
RubricaGroupList
rubrica_item_get_groups (RubricaItem *item)
{
  g_return_val_if_fail(IS_RUBRICA_ITEM(item), NULL);

  return (RubricaGroupList) item->groups;
}


/* get a group 
 */
RubricaGroup*
rubrica_item_get_group(RubricaGroupList groups)
{
//  g_return_val_if_fail(IS_RUBRICA_ITEM(item), NULL);

  return (RubricaGroup*) groups->data;
}


/* get group label 
 */
gchar* 
rubrica_item_get_group_label(RubricaGroup* group)
{
//  g_return_val_if_fail(IS_RUBRICA_ITEM(item), NULL);

  return (gchar *) group->label;
}


/* get group type 
 */
RubricaGroupType
rubrica_item_get_group_type(RubricaGroup* group)
{
//  g_return_val_if_fail(IS_RUBRICA_ITEM(item), NO_GROUP);

  return (RubricaGroupType) group->type;
}

gboolean          
rubrica_item_belong_to_group (RubricaItem *item,
				   RubricaGroupType type)
{
  RubricaGroupList groups;

  g_return_val_if_fail(IS_RUBRICA_ITEM(item), FALSE);

  if (type == ALL_GROUP)
    return TRUE;

  groups = rubrica_item_get_groups(item);  
  for (; groups; groups = g_list_next(groups))
    {
      RubricaGroup* group;

      group = rubrica_item_get_group(groups);

      if (!group) 
	return FALSE;
      
      g_print("\n\ntype: %d"
	      "\ngroup label: %s"
	      "\ngroup type: %d"
	      "\n-------------------------",
	      type, group->label, group->type);
      if (rubrica_item_get_group_type(group) == type)
	return TRUE;
    }  
  
  return FALSE;
}



/* set item's father to father 
 */
void             
rubrica_item_set_father (RubricaItem *item, RubricaDoc* father)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  item->father = father;
}


/* set item's deleted flag 
 */ 
void 
rubrica_item_set_deleted(RubricaItem *item, gboolean state)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  item->deleted = (gboolean) state;
}

void 
rubrica_item_set_deletable (RubricaItem* item, gboolean bool)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  item->deletable = bool;  
}

void 
rubrica_item_set_marked (RubricaItem* item, gboolean bool)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  item->marked = bool;
}

void 
rubrica_item_set_modification_time (RubricaItem* item, time_t time)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  item->changed = time;
}

/* set item's rank 
 */
void 
rubrica_item_set_rank (RubricaItem *item, RubricaRankType rank)
{
  g_return_if_fail(item != NULL);

   RUBRICA_ITEM(item)->marked = (RubricaRankType) rank;
}

void 
rubrica_item_set_position(RubricaItem *item, gint pos)
{
  g_return_if_fail(IS_RUBRICA_ITEM(item));

  item->position = pos;
}


void 
rubrica_item_free_address (RubricaCardAddress* address)
{
  g_return_if_fail(address != NULL);

  rubrica_free_string(address->street);
  rubrica_free_string(address->number);
  rubrica_free_string(address->zip);
  rubrica_free_string(address->city);
  rubrica_free_string(address->province);
  rubrica_free_string(address->state);
  rubrica_free_string(address->country);     
}


void 
rubrica_item_free_addresses(RubricaCardAddresses addresses)
{
  GList* link;

  for (; addresses; addresses = g_list_next(addresses))
    {
      RubricaCardAddress* address;
     
      link = addresses;
      addresses = g_list_remove_link(addresses, link);
      if (addresses->data)
	{
	  address = (RubricaCardAddress*) addresses->data;

	  rubrica_item_free_address(address);	  
	}

      g_list_free_1(link);
    }
}


void 
rubrica_item_free_item(RubricaItem *item)
{
  g_object_unref(item);
}
