/*
*  Rubrica
*  file: ui.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <config.h>

#include <gnome.h>
#include <gconf/gconf-client.h>

#include "ui.h"
#include "prefer.h"
#include "callbacks.h"
#include "utils.h"



GtkWidget *
rubrica_ui_default_box_new(RubricaBoxType type)
{
  return (GtkWidget *) rubrica_ui_box_new(type, FALSE, 0, 6);
}


GtkWidget *
rubrica_ui_label_new(gchar *str, GtkWidget *target) 
{ 
  GtkWidget *label; 

  g_return_val_if_fail(str != NULL, NULL);
  
  if (target)
    {
      label = gtk_label_new_with_mnemonic(str); 
      gtk_label_set_mnemonic_widget(GTK_LABEL(label), target);
    }
  else
    label = gtk_label_new(str);  

  gtk_widget_show(label); 
  
  return (GtkWidget *) label; 
} 

GtkWidget*
rubrica_ui_packed_check_button_new(GtkWidget *box, gchar *str) 
{
  GtkWidget *check;  
  
  g_return_val_if_fail(box != NULL, NULL);

  check =  gtk_check_button_new_with_mnemonic(str);  
  gtk_container_set_border_width (GTK_CONTAINER (check), 12);    

  gtk_box_pack_start(GTK_BOX(box), check, FALSE, FALSE, 0);

  gtk_widget_show(check);  
  
  return (GtkWidget *) check;  
}




void rubrica_ui_set_default_icon(GtkWidget *app)
{  
  GdkPixbuf *icon;

  icon = rubrica_ui_pixbuf_new("rub.png");
  if (!icon)
    g_print("\nRubrica: setting default icon failed");

  gtk_window_set_icon(GTK_WINDOW(app), icon);
  g_object_unref (icon);
}

/**
 * rubrica_ui_hig_frame_new
 * @str: the label to show
 *
 * make a new frame with the label "str", according to HIG
 * (no frame shadow and 12 pixel border)
 *
 * return a pointer to a new frame with "str" label
 */

GtkWidget *
rubrica_ui_hig_frame_new(gchar *str)
{
  GtkWidget *frame;
  GtkWidget *label;

  g_return_val_if_fail(str != NULL, NULL);
  
  frame = gtk_frame_new(NULL);
  gtk_frame_set_shadow_type(GTK_FRAME(frame), GTK_SHADOW_NONE);

  //* from HIG: put 12 pixel between dialog border and frame label /
  gtk_container_set_border_width(GTK_CONTAINER(frame), 12);            

  label = gtk_label_new(str);   
  gtk_label_set_use_markup(GTK_LABEL(label), TRUE);
  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
  gtk_frame_set_label_widget(GTK_FRAME(frame), label);  

  gtk_widget_show(frame);
  gtk_widget_show(label);

  return (GtkWidget *) frame;  
}

/**
 * rubrica_ui_frame_new
 * @str: the label to show
 *
 * make a new frame with the label "str"
 *
 * return a pointer to a new frame with "str" label
 */
GtkWidget *
rubrica_ui_frame_new(gchar *str)
{
  GtkWidget *frame;
  GtkWidget *label;

  g_return_val_if_fail(str != NULL, NULL);

  frame = gtk_frame_new(NULL);
  
  label =  gtk_label_new(_(str));
  gtk_container_set_border_width(GTK_CONTAINER(frame), 2);          

  gtk_frame_set_label_widget(GTK_FRAME(frame), label);
  gtk_misc_set_padding (GTK_MISC (label), 5, 0);

  gtk_widget_show(frame);
  gtk_widget_show(label);

  return (GtkWidget *) frame;  
}



/**
 * rubrica_ui_box_new
 * @type: the type of the new box
 *        HBOX an horizontal box
 *        VBOX a vertical box
 * @homogeneous:
 * @spacing:
 * @border_widtg:
 *
 * create a new vbox/hbox 
 *
 * return: the pointer to new box. The new widget is shown automatically 
 *         (don't call gtk_widget_show())
 */

GtkWidget *
rubrica_ui_box_new(RubricaBoxType type, gboolean homogeneous, 
		   gint spacing, guint border_width)
{
  GtkWidget *box;

  if (type == VBOX)
    box = gtk_vbox_new (homogeneous, spacing);     
  else
    box = gtk_hbox_new (homogeneous, spacing);

  gtk_container_set_border_width (GTK_CONTAINER (box), border_width);
  
  gtk_widget_show (box); 
  
  return (GtkWidget *) box;
}



GtkWidget *
rubrica_ui_popup_image_new(gchar *pixfile)
{
  GtkWidget *image = NULL;
  gchar *imagefile = NULL;

  if (!pixfile || !pixfile[0])
    return gtk_image_new ();

  imagefile = rubrica_utils_find_pixmap_file(pixfile);
  
  if (!imagefile)
    {
      g_warning (_("Couldn't find pixmap file: %s"), pixfile);
      
      return gtk_image_new ();
    }
  
  image = gtk_image_new_from_file(imagefile);
  g_free(imagefile);
  
  return (GtkWidget *) image;
}

GtkWidget *
rubrica_ui_image_new(gchar *pixfile)
{
  GtkWidget *image = NULL;
  gchar *imagefile = NULL;

  if (!pixfile || !pixfile[0])
    return gtk_image_new ();

  imagefile = rubrica_utils_find_pixmap_file(pixfile);
  
  if (!imagefile)
    {
      g_warning (_("Couldn't find pixmap file: %s"), pixfile);
      
      return gtk_image_new ();
    }
  
  image = gtk_image_new_from_file(imagefile);
  gtk_widget_show(image);
  g_free(imagefile);
  
  return (GtkWidget *) image;
}


GdkPixbuf*
rubrica_ui_pixbuf_new(gchar *filename)
{
  GdkPixbuf *pix = NULL;
  gchar *imagefile = NULL;
  GError *error = NULL;

  g_return_val_if_fail(filename != NULL, NULL);
  
  if ((g_ascii_strcasecmp(filename, "") == 0)) return NULL;

  imagefile = rubrica_utils_find_pixmap_file(filename);
      
  if (!imagefile)
    {
      g_warning (_("Couldn't find pixmap file: %s"), filename);
      return NULL;
    }
  
  pix = gdk_pixbuf_new_from_file(imagefile, NULL);
  if (!pix)
    {
      fprintf (stderr, "Failed to load pixbuf file: %s - error: %s\n", 
	       filename, error->message);
      g_error_free (error);
    }  

  g_free(imagefile);
  
  return  pix;
}

GtkWidget*
rubrica_ui_scrolled_window_new(void)
{  
  GtkWidget *scrollwin;
  
  scrollwin = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrollwin), 
				      GTK_SHADOW_ETCHED_IN);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollwin),
				 GTK_POLICY_AUTOMATIC, 
				 GTK_POLICY_AUTOMATIC);

  gtk_widget_show(scrollwin);

  return (GtkWidget *) scrollwin;
}


GtkWidget *rubrica_ui_option_menu_new(GtkWidget* opt_menu,
				      RubricaOptionType *option_list)
{
  RubricaOptionType *list = option_list;
  GtkWidget *optmenu = NULL;
  GtkWidget *optitem;
  GtkWidget *menu;

  menu    = gtk_menu_new();
  if (!opt_menu)
    {
      optmenu = gtk_option_menu_new();
      gtk_option_menu_set_menu(GTK_OPTION_MENU(optmenu), menu);      
    }
  else
    gtk_option_menu_set_menu(GTK_OPTION_MENU(opt_menu), menu);      


  while (list->pixmap)
    {
      GtkWidget *image;

      if ((g_ascii_strcasecmp(list->label, "-") == 0))
	optitem = gtk_separator_menu_item_new();
      else
	optitem = gtk_image_menu_item_new_with_mnemonic(_(list->label));

      if ((g_ascii_strcasecmp(list->pixmap, "") != 0))
	{
	  image = rubrica_ui_image_new(list->pixmap);
	  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(optitem), image);
	}      

      gtk_container_add(GTK_CONTAINER(menu), optitem);
      gtk_widget_show(optitem);

      list++;
    }  

  gtk_widget_show(menu);
  if (optmenu)
    gtk_widget_show(optmenu);

  return (GtkWidget *) optmenu;
}



GtkWidget *rubrica_ui_popup_menu_new(RubricaMenu *rubrica_menu)
{
  RubricaMenu *list = rubrica_menu;
  GConfClient *client;
  GtkWidget *menu;
  GtkWidget *submenu;
  GtkWidget *image;
  GtkWidget *menuitem = NULL;
  GSList *group = NULL;

  menu    = gtk_menu_new();
  client  = gconf_client_get_default();

  while (list->type != RM_END)
    {
      RubricaMenuType type = list->type;

      switch (type)
	{  
	case RM_SEPARATOR:
	  menuitem = gtk_separator_menu_item_new();
	  break;
	  
	case RM_HEADER:
	  menuitem = gtk_menu_item_new_with_mnemonic(_(list->label));
	  break;

	case RM_ITEM:
	  if (list->pixtype == RMP_STOCK)
	    menuitem = gtk_image_menu_item_new_from_stock(list->pixmap, NULL);
	  else
	    {
	      menuitem = gtk_image_menu_item_new_with_mnemonic(_(list->label));
	      
	      if (list->pixtype == RMP_FILE)
		{
		  image = rubrica_ui_popup_image_new(list->pixmap);
		  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), 
						image);
		}
	    }
	  if (list->callback)
	    g_signal_connect(G_OBJECT(menuitem), "activate", 
			     G_CALLBACK(list->callback), list->data);
	  break;

	case RM_RADIO_ITEM:  
	  menuitem = gtk_radio_menu_item_new_with_mnemonic(group, 
							   _(list->label));

	  group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(menuitem));
	  if (g_ascii_strcasecmp(list->data, RUBRICA_CRYPT_ALL_DATA) == 0)
	    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
					   TRUE);	  	  
	  if (list->callback)
	    g_signal_connect(G_OBJECT(menuitem), "toggled", 
			     G_CALLBACK(list->callback), list->data);
	  break;

	case RM_CHECK_ITEM:
	  menuitem = gtk_check_menu_item_new_with_mnemonic(_(list->label));

	  gconf_client_notify_add(client, list->data, 
				  rubrica_ui_update_popup_item, menuitem,
				  NULL, NULL);
	  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), 
					 gconf_client_get_bool(client, 
							       list->data, 
							       NULL));

	  if (list->callback)
	    g_signal_connect(G_OBJECT(menuitem), "toggled", 
			     G_CALLBACK(list->callback), list->data);
	  break;
	  
	case RM_ICONED_SUBTREE:
	  if (list->pixtype == RMP_STOCK)
	    menuitem = gtk_image_menu_item_new_from_stock(list->pixmap, NULL);
	  else
	    {
	      menuitem = gtk_image_menu_item_new_with_mnemonic(_(list->label));
	      
	      if (list->pixtype == RMP_FILE)
		{
		  image = rubrica_ui_popup_image_new(list->pixmap);
		  gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(menuitem), 
						image);
		}
	    }

	  submenu = rubrica_ui_popup_menu_new(list->more);
	  gtk_menu_item_set_submenu (GTK_MENU_ITEM(menuitem), submenu);
	  break;

	case RM_SUBTREE:
	  menuitem = gtk_menu_item_new_with_mnemonic(_(list->label));
	  submenu  = rubrica_ui_popup_menu_new(list->more);
	  gtk_menu_item_set_submenu (GTK_MENU_ITEM(menuitem), submenu);
	  break;

	case RM_END:
	default:
	  break; 
	}

      gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuitem);      
      list++;
    }  
  
  return (GtkWidget *) menu;
}

/*  Text 
*/
GtkWidget*
rubrica_ui_text_view_new(void)
{
  GtkWidget *text_view;
  
  text_view = gtk_text_view_new ();
  gtk_text_view_set_editable (GTK_TEXT_VIEW (text_view), TRUE);
  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (text_view), GTK_WRAP_WORD);
  gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (text_view), TRUE);

  gtk_widget_show(text_view);
  
  return (GtkWidget *) text_view;
}


void 
rubrica_ui_clean_text_view(GtkWidget *text_view)
{
  GtkTextIter start, end;
  GtkTextBuffer *text_buffer;  
  
  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));
  
  gtk_text_buffer_get_bounds(text_buffer, &start, &end);
  gtk_text_buffer_delete(text_buffer, &start, &end);
}


void 
rubrica_ui_set_text_in_view(GtkWidget *text_view, gchar *buffer)
{
  GtkTextIter start, end;
  GtkTextBuffer *text_buffer;  

  g_return_if_fail(g_utf8_validate(buffer, -1, NULL));
  
  text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (text_view));  
  gtk_text_buffer_get_bounds(text_buffer, &start, &end);

  gtk_text_buffer_insert(text_buffer, &start, buffer, -1);  
}


/*  Tree/List
*/
GtkWidget*
rubrica_ui_tree_view_new_with_model(GtkTreeModel* model)
{
  GtkWidget *tree_view;
  
  tree_view = gtk_tree_view_new_with_model(model);

  gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(tree_view), TRUE);
  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree_view), TRUE);

  g_object_unref(G_OBJECT(model));

  gtk_widget_show(tree_view);

  return (GtkWidget *) tree_view; 
}



GtkWidget *
rubrica_ui_pack_in_frame_tree_view_new_with_model(GtkWidget *frame, 
						  GtkTreeModel* model)
{
  GtkWidget *scrollwin;
  GtkWidget *tree_view;

  scrollwin = rubrica_ui_scrolled_window_new();
  tree_view = rubrica_ui_tree_view_new_with_model(model);
  gtk_container_add(GTK_CONTAINER(scrollwin), tree_view);

  gtk_container_add(GTK_CONTAINER(frame), scrollwin);
  
  return (GtkWidget *) tree_view;   
}


GtkWidget *
rubrica_ui_packed_tree_view_new_with_model(GtkWidget *box, 
						GtkTreeModel* model)
{
  GtkWidget *scrollwin;
  GtkWidget *tree_view; 

  scrollwin = rubrica_ui_scrolled_window_new(); 
  tree_view = rubrica_ui_tree_view_new_with_model(model); 
  gtk_container_add(GTK_CONTAINER(scrollwin), tree_view);

  gtk_box_pack_start(GTK_BOX(box), scrollwin, TRUE, TRUE, 2);
  
  return (GtkWidget *) tree_view; 
}

void rubrica_ui_update_popup_item(GConfClient *client, guint cnxn_id, 
				  GConfEntry *entry, gpointer data)
{
  GtkWidget *menuitem = data;
  gboolean bool;

  bool = gconf_value_get_bool(entry->value);
  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem), bool); 
}






























































